COMMENT
ihw.mod
based on
isin.mod
but omits isin.mod's phase parameter phi.

Delivers num cycles of half-wave rectified sinusoidal current
that start either at t == del
or when an external event arrives at time t = tx.
External or endogenous activating events that arrive
while the generator is on are ignored.

Current is
i = amp*hw(0.002*PI*f*(t - del))
where
amp is peak ampltude in nA (may be + or -)
f is frequency in Hz
t is time in ms
hw(theta) = sin(theta) if sin(theta)>=0
            0 if sin(theta)<0
12/11/2013 NTC
ENDCOMMENT

NEURON {
  POINT_PROCESS Ihw
  RANGE del, num, f, amp, i
  ELECTRODE_CURRENT i
}

UNITS {
  (nA) = (nanoamp)
  PI = (pi) (1)
  (Radian) = (1)
}

PARAMETER {
  del = 0 (ms)          : time at which current starts
  num = 0 (1) <0, 1e9>  : number of cycles
  f (Hz) <0, 1e9>       : frequency in Hz
  amp (nA)
}

ASSIGNED {
  i (nA)
  on (1)
  tstart (ms)
}

INITIAL {
  i = 0
  on = 0
  if (num>0) {
    net_send(del, 1)
:    tstart = del
  }
}

COMMENT
FUNCTION fsin(t (ms)) {
  fsin = sin((0.001)*2*PI*f*(t - del))
}

FUNCTION hw(t (ms)) {
  hw = fsin(t)
  if (hw<0) {
    hw = 0
  }
}
ENDCOMMENT

FUNCTION hw(th (Radian)) {
  hw = sin(th)
  if (hw<0) {
    hw = 0
  }
}

BREAKPOINT {
  i = on*amp*hw((0.001)*2*PI*f*(t - del))
}

COMMENT
NET_RECEIVE (w) {
  if (flag==1) {
    if (on == 0) {
      : turn it on
      on = 1
      : 1 cycle lasts 1000/f ms
      : num positive half cycles are completed in (num-1)+0.5 cycles
      : but give it a little extra time to avoid problems with roundoff error
      net_send(((num-1)+0.55)*(1000)/f, 1)
    } else {
      : turn it off
      on = 0
    }
  }
}
ENDCOMMENT

NET_RECEIVE (w) {
  if (on == 0) {
    if ((flag == 0) || (flag == 1)) { : an "activating event"
      tstart = t
      on = 1 : turn it on
      : 1 cycle lasts 1000/f ms
      : num positive half cycles are completed in (num-1)+0.5 cycles
      : but give it a little extra time to avoid problems with roundoff error
      net_send(((num-1)+0.55)*(1000)/f, 2)
    }
  } else { : already on, ignore "activating events"
    if (flag==2) {
      on = 0
    }
  }
}

